/*	
	Copyright  1991-1995 by TopSoft Inc.  All rights reserved.

	You may distribute this file under the terms of the TopSoft
	Artistic License, accompanying this package.
	
	This file was developed by George (ty) Tempel in connection with TopSoft, Inc..
	See the Modification History for more details.

Product
	About Box

FILE
	ABUEnvDragMgr.h

NAME
	ABUEnvDragMgr.h, part of the ABox project source code,
	a utility class, mixed into other classes, responsible for 
	handling the AboutBox DragMgr stuff.

DESCRIPTION
	This file contains defines for the about box modules.
	
DEVELOPED BY
	George (ty) Tempel				netromancr@aol.com
	All code in this file, and its associated header file was
	Created by George (ty) Tempel in connection with the TopSoft, Inc.
	"FilterTop" application development, except where noted.

CARETAKER - George (ty) Tempel <netromancr@aol.com>
 	Please consult this person for any changes or suggestions to this file.

MODIFICATION HISTORY

	dd mmm yy	-	xxx	-	patchxx: description of patch
	27 June 94	-	ty	-	Initial Version Created
	20-july-94	-	ty	-	initial version released
	28-july-94	-	ty	-	1.0.6--fixes to watch for Region errors
								during drag process
	23-may-95	-	ty	-	changes for compatibility with the CodeWarrior CW6
							release and the associated Universal Headers from Apple:
							most methods that returned references now have "Ref" at
							the end of their methods names to prevent possible collisions
							with datatypes and classes of the same name (older versions
							of the compiler didn't have a problem with this).

*/

/*===========================================================================*/

/*========== Exclusion Macros ============*/

#pragma	once

#ifndef	_ABUEnvDragMgr_
#define	_ABUEnvDragMgr_


/*============ Header Files ==============*/

#include	"ABUEnv.h"

#ifndef __DRAG__
#include "Drag.h"
#endif

/*=========== External Linkage ===========*/

/*================ Macros ================*/

/*============== Constants ===============*/

#define		kABdefaultClippingType		'TEXT'
#define		kABdefaultClippingCreator	'ttxt'
#define		kABdefaultFlavorFlags		((long)0)

/*================ Enums =================*/

/*=============== Structs ================*/

/*=============== Typedefs ===============*/


/*=========== Class Definitions ==========*/

class	ABUEnvDragMgr : ABUEnv
{
	public:
								ABUEnvDragMgr(void);
		virtual					~ABUEnvDragMgr(void);
	
		virtual	Boolean			IsPresent(void);
	
		virtual	OSErr			Initialize(void);
		virtual	OSErr			Begin(void);
		virtual	OSErr			End(void);
		
		//	1.0.6 ty...changed to OSErr OutlineRegion()
		virtual	OSErr			OutlineRegion(void);
		
		virtual OSErr			LocalSendProc(FlavorType theType,
										void *dragSendRefCon,
										ItemReference theItemRef,
										DragReference theDragRef);

		static pascal OSErr		SendProc(FlavorType theType,
										void *dragSendRefCon,
										ItemReference theItemRef,
										DragReference theDragRef);

		static	OSErr			GetDropLocationDirectory( AEDesc *dropLocation,
															long *directoryID,
															short *volumeID);
													
		virtual	OSErr			MakeDragRegion (void);
		
								Boolean IsDragInitialized(void) const { return this->DragInitializedRef() == true; }
								Boolean IsntDragInitialized(void) const { return ! this->IsDragInitialized(); }

								Boolean HasDragRef(void) const { return this->DragRef() != NULL; }
								Boolean DoesntHaveDragRef(void) const { return ! this->HasDragRef(); }

								Boolean HasDragRegionRef(void) const { return this->DragRegionRef() != NULL; }
								Boolean DoesntHaveDragRegionRef(void) const { return ! this->HasDragRegionRef(); }

								Boolean HasDragUPPRef(void) const { return this->DragUPPRef() != NULL; }
								Boolean DoesntHaveDragUPPRef(void) const { return ! this->HasDragUPPRef(); }

								Boolean HasDragMgrEventRef(void) const { return this->DragMgrEventRef() != NULL; }
								Boolean DoesntHaveDragMgrEventRef(void) const { return ! this->HasDragMgrEventRef(); }
								
	protected:
		
			Boolean				mDragInitialized;
			DragReference		mDragReference;
			ItemReference		mItemReference;
			RgnHandle			mDragRegion;
			Rect				mDragRect;
			DragSendDataProc	mDragUPP;
			EventRecord*		mDragMgrEvent;

			Boolean&			DragInitializedRef(void) const { return this->mDragInitialized; }
			DragReference&		DragRef(void) const { return this->mDragReference; }
			ItemReference&		ItemRef(void) const { return this->mItemReference; }
			RgnHandle&			DragRegionRef(void) const { return this->mDragRegion; }
			Rect&				DragRectRef(void) const { return this->mDragRect; }
			EventRecord*&		DragMgrEventRef(void) const { return this->mDragMgrEvent; }

			DragSendDataProc&	DragUPPRef(void) const { return this->mDragUPP; }
			
	private:
		
};


/*========== Function Prototypes =========*/


#endif	// _ABUEnvDragMgr_